<?php declare(strict_types = 1);

namespace Drupal\c2pa_sign\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Jrglasgow\C2paTool\Tool;

/**
 * Configure C2PA Sign settings for this site.
 */
final class SettingsForm extends ConfigFormBase {


  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'c2pa_sign_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames(): array {
    return ['c2pa_sign.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $config = $this->config('c2pa_sign.settings');

    $form['site'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Site Information'),
      '#tree' => TRUE,
    ];
    $form['site']['name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Site/Org Name'),
      '#description' => $this->t('What site or organization name do you want to appear in the manifests when signing? If left blank the <a href="@url" target="_blank">system site name</a> "%site_name" will be used..', [
        '%site_name' => \Drupal::config('system.site')->get('name'),
        '@url' => Url::fromRoute('system.site_information_settings')->toString(),
      ]),
      '#default_value' => $config->get('site')['name'] ?? '',
    ];

    $form['site']['claim_generator'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Claim Generator'),
      '#description' => $this->t('When a manifest is created a claim_generator is added - A User-Agent string formatted as per <a href="http://tools.ietf.org/html/rfc7231#section-5.5.3" target="_blank">http://tools.ietf.org/html/rfc7231#section-5.5.3</a>, for including the name and version of the claims generator that created the claim. If left blank the claim generator for the c2patool library will be used.'),
      '#default_value' => $config->get('site')['claim_generator'] ?? '',
    ];

    $form['certificate'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Certificate'),
    ];

    $certs = c2pa_sign_get_certs($config->get('certificate_file_directory'));
    // look through the valid certs and determine if one is in the environment variable
    $in_env = FALSE;
    $placeholder = '/path/to/certificate/directory';
    $required = TRUE;
    foreach ($certs['certificates'] AS $this_cert) {
      if ($this_cert->uri == 'ENVIRONMENT_VARIABLE') {
        $placeholder = $this->t('The environment variables contain a valid certificate.');
        $required = FALSE;
      }
    }
    $form['certificate']['certificate_file_directory'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Certificate file directory'),
      '#description' => $this->t('The directory in which you keep your certificates and keys. When looking to sign media this directory will be searched for valid certificate key pairs. A key pair will consist of 2 files with the same name but different extensions, i.e. cert-@year.pem and cert-@year.key. Only key pairs that are complete will be used. If there are multiple pairs the certificate that expires last will be used. Certificates MUST fulfill all requirements in the <a href="https://opensource.contentauthenticity.org/docs/manifest/signing-manifests/#certificates" target="_blank">documentation</a>.', ['@year' => date('Y')]),
      '#default_value' => $config->get('certificate_file_directory'),
      '#required' => $required,
      '#placeholder' => $placeholder,
    ];

    if (!empty($certs['cert_to_use']) && is_object($certs['cert_to_use'])) {
      $form['certfiicate']['cert_info'] = c2pa_sign_certificate_info($certs['cert_to_use']);
    }
    $logger = $this->logger('c2pa_sign');
    $tool = new Tool($logger);
    if (!empty($config->get('c2patool_binary_location')) && file_exists($config->get('c2patool_binary_location')) && is_executable($config->get('c2patool_binary_location'))) {
      $tool->setBinary($config->get('c2patool_binary_location'));
    }
    $form['c2patool'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('c2patool'),
    ];
    $form['c2patool']['c2patool_binary_location'] = [
      '#type' => 'textfield',
      '#title' => $this->t('c2patool location'),
      '#description' => $this->t('The location on the system of the <a href="https://github.com/contentauth/c2patool" target="_blank">c2patool binary which can be found on Github</a>.'),
      '#default_value' => $tool->getBinary(),
    ];

    if ($tool->getBinary()) {
      $form['c2patool']['c2patool_info'] = [
        '#markup' => t('c2patool version: %version.', ['%version' => $tool->getBinaryVersion()]),
      ];
    }

    $form['assertions'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Assertions'),
      '#description' => $this->t('What actions should trigger assertions being added to media assets?'),
      '#tree' => TRUE,
    ];

    $form['assertions']['upload'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Assert that files were uploaded to the site.'),
      '#description' => $this->t('Integration with the FileWidget will cause assertions to be added upon upload.'),
      '#default_value' => $config->get('assertions')['upload'] ?? TRUE,
    ];

    $form['assertions']['publish'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Assert that files were published.'),
      '#description' => $this->t('Integration with the Node module will cause assertions to be added to files upon publication.'),
      '#default_value' => $config->get('assertions')['publish'] ?? TRUE,
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state): void {
    $certs = c2pa_sign_get_certs($form_state->getValue('certificate_file_directory'));
    if (empty($certs['cert_to_use'])) {
      $form_state->setErrorByName('certificate_file_directory', $this->t('No valid cert/key pairs in the directory %directory.', ['%directory' => $form_state->getValue('certificate_file_directory')]));

    }
    if (!empty($certs['errors'])) {
      foreach ($certs['errors'] AS $error) {
        if (isset($error['text']) && !empty($error['text'])) {
          $form_state->setErrorByName('certificate_file_directory', $error['text'], $error['args'] ?? []);
          $this->messenger()->addWarning($error['text'], $error['args'] ?? []);
        }
      }
    }

    // validate c2patool binary
    $binary_location = $form_state->getValue('c2patool_binary_location');
    if (empty($binary_location)) {
      $this->messenger()->addWarning('c2patool binary location is empty.');
      $this->searchBinary($form_state);
      $this->messenger()->addWarning('c2patool binary location is empty.');
      if (empty($form_state->getValue('c2patool_binary_location'))) {
        $form_state->setErrorByName('c2patool_binary_location', $this->t('c2patool binary location is required.'));
      }
    }
    else if (!file_exists($binary_location)) {
      $this->messenger()->addWarning($this->t('c2patool binary location (%location) does not exist.', ['%location' => $binary_location]));
      $this->searchBinary($form_state);
      if ($binary_location == $form_state->getValue('c2patool_binary_location')) {
        // the binary location didn't change
        $form_state->setErrorByName('c2patool_binary_location', $this->t('c2patool binary location is required and MUST exist.'));
      }
    }
    else if (!is_executable($binary_location)) {
      $this->messenger()->addWarning($this->t('c2patool binary location (%location) is not executable.', ['%location' => $binary_location]));
      $this->searchBinary($form_state);
      if ($binary_location == $form_state->getValue('c2patool_binary_location')) {
        // the binary location didn't change
        $form_state->setErrorByName('c2patool_binary_location', $this->t('c2patool binary location is required and MUST be executable.'));
      }
    }

    parent::validateForm($form, $form_state);
  }

  /**
   * Search for the c2patool binary ans set it if found
   *
   * @param $form_state
   *
   * @return void
   */
  protected function searchBinary($form_state) {
    $tool = new Tool($this->logger('c2pa_sign'));
    $binary_location = $tool->getBinary();
    if (!empty($binary_location)) {
      $form_state->setValue('c2patool_binary_location', $binary_location);
      \Drupal::messenger()->addStatus('c2patool binary found at %location', ['%location' => $binary_location]);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $config = $this->config('c2pa_sign.settings');
    // remove extraneous form values
    $form_state->cleanValues();
    // set all form values
    foreach ($form_state->getValues() as $key => $value) {
      $config->set($key, $value);
    }
    // save the config
    $config->save();
    parent::submitForm($form, $form_state);
    parent::submitForm($form, $form_state);
  }

}
